/*
    SPDX-FileCopyrightText: 2014 Jan Grulich <jgrulich@redhat.com>

    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
*/

#include "Debug.h"

Q_LOGGING_CATEGORY(PM_CONFIGURE_PRINTER, "pm.configure.printer");
