/****************************************************************************
** Meta object code from reading C++ file 'CommandHelpers.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../print-manager-6.6.90/src/libkcups/CommandHelpers.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'CommandHelpers.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15PrinterCommandsE_t {};
} // unnamed namespace

template <> constexpr inline auto PrinterCommands::qt_create_metaobjectdata<qt_meta_tag_ZN15PrinterCommandsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PrinterCommands",
        "QML.Element",
        "auto",
        "QML.Singleton",
        "true",
        "error",
        "",
        "lastError",
        "errorTitle",
        "errorMsg",
        "removeDone",
        "saveDone",
        "forceRefresh",
        "pauseDone",
        "resumeDone",
        "defaultDone",
        "sharedDone",
        "testDone",
        "cleanDone",
        "acceptDone",
        "pausePrinter",
        "printerName",
        "resumePrinter",
        "removePrinter",
        "setDefault",
        "setShared",
        "isClass",
        "shared",
        "setAcceptingJobs",
        "accept",
        "printTestPage",
        "printSelfTestPage",
        "cleanPrintHeads",
        "savePrinter",
        "QVariantMap",
        "saveArgs",
        "PPDType",
        "Manual",
        "Auto",
        "Custom"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'error'
        QtMocHelpers::SignalData<void(int, const QString &, const QString &)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 },
        }}),
        // Signal 'removeDone'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'saveDone'
        QtMocHelpers::SignalData<void(bool)>(11, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Signal 'pauseDone'
        QtMocHelpers::SignalData<void()>(13, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'resumeDone'
        QtMocHelpers::SignalData<void()>(14, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'defaultDone'
        QtMocHelpers::SignalData<void()>(15, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sharedDone'
        QtMocHelpers::SignalData<void()>(16, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'testDone'
        QtMocHelpers::SignalData<void()>(17, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cleanDone'
        QtMocHelpers::SignalData<void()>(18, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'acceptDone'
        QtMocHelpers::SignalData<void()>(19, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'pausePrinter'
        QtMocHelpers::MethodData<void(const QString &)>(20, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Method 'resumePrinter'
        QtMocHelpers::MethodData<void(const QString &)>(22, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Method 'removePrinter'
        QtMocHelpers::MethodData<void(const QString &)>(23, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Method 'setDefault'
        QtMocHelpers::MethodData<void(const QString &)>(24, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Method 'setShared'
        QtMocHelpers::MethodData<void(const QString &, bool, bool)>(25, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 }, { QMetaType::Bool, 26 }, { QMetaType::Bool, 27 },
        }}),
        // Method 'setShared'
        QtMocHelpers::MethodData<void(const QString &, bool)>(25, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 21 }, { QMetaType::Bool, 26 },
        }}),
        // Method 'setAcceptingJobs'
        QtMocHelpers::MethodData<void(const QString &, bool)>(28, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 }, { QMetaType::Bool, 29 },
        }}),
        // Method 'setAcceptingJobs'
        QtMocHelpers::MethodData<void(const QString &)>(28, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Method 'printTestPage'
        QtMocHelpers::MethodData<void(const QString &, bool)>(30, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 }, { QMetaType::Bool, 26 },
        }}),
        // Method 'printSelfTestPage'
        QtMocHelpers::MethodData<void(const QString &)>(31, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Method 'cleanPrintHeads'
        QtMocHelpers::MethodData<void(const QString &)>(32, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Method 'savePrinter'
        QtMocHelpers::MethodData<void(const QString &, const QVariantMap &, bool)>(33, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 }, { 0x80000000 | 34, 35 }, { QMetaType::Bool, 26 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'PPDType'
        QtMocHelpers::EnumData<enum PPDType>(36, 36, QMC::EnumIsScoped).add({
            {   37, PPDType::Manual },
            {   38, PPDType::Auto },
            {   39, PPDType::Custom },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<PrinterCommands, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject PrinterCommands::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PrinterCommandsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PrinterCommandsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15PrinterCommandsE_t>.metaTypes,
    nullptr
} };

void PrinterCommands::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PrinterCommands *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->error((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 1: _t->removeDone(); break;
        case 2: _t->saveDone((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->pauseDone(); break;
        case 4: _t->resumeDone(); break;
        case 5: _t->defaultDone(); break;
        case 6: _t->sharedDone(); break;
        case 7: _t->testDone(); break;
        case 8: _t->cleanDone(); break;
        case 9: _t->acceptDone(); break;
        case 10: _t->pausePrinter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->resumePrinter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->removePrinter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->setDefault((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->setShared((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 15: _t->setShared((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 16: _t->setAcceptingJobs((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 17: _t->setAcceptingJobs((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->printTestPage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 19: _t->printSelfTestPage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 20: _t->cleanPrintHeads((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 21: _t->savePrinter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PrinterCommands::*)(int , const QString & , const QString & )>(_a, &PrinterCommands::error, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PrinterCommands::*)()>(_a, &PrinterCommands::removeDone, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PrinterCommands::*)(bool )>(_a, &PrinterCommands::saveDone, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PrinterCommands::*)()>(_a, &PrinterCommands::pauseDone, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PrinterCommands::*)()>(_a, &PrinterCommands::resumeDone, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PrinterCommands::*)()>(_a, &PrinterCommands::defaultDone, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (PrinterCommands::*)()>(_a, &PrinterCommands::sharedDone, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (PrinterCommands::*)()>(_a, &PrinterCommands::testDone, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (PrinterCommands::*)()>(_a, &PrinterCommands::cleanDone, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (PrinterCommands::*)()>(_a, &PrinterCommands::acceptDone, 9))
            return;
    }
}

const QMetaObject *PrinterCommands::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PrinterCommands::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PrinterCommandsE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int PrinterCommands::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void PrinterCommands::error(int _t1, const QString & _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void PrinterCommands::removeDone()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void PrinterCommands::saveDone(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void PrinterCommands::pauseDone()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void PrinterCommands::resumeDone()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void PrinterCommands::defaultDone()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void PrinterCommands::sharedDone()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void PrinterCommands::testDone()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void PrinterCommands::cleanDone()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void PrinterCommands::acceptDone()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
