/****************************************************************************
** Meta object code from reading C++ file 'printermanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../print-manager-6.6.90/src/kcm/printermanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'printermanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14PrinterManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto PrinterManager::qt_create_metaobjectdata<qt_meta_tag_ZN14PrinterManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PrinterManager",
        "requestError",
        "",
        "errorMessage",
        "serverSettingsChanged",
        "serverStopped",
        "serverStarted",
        "remotePrintersLoaded",
        "recommendedDriversLoaded",
        "ppdLoaded",
        "QVariantMap",
        "printerPPD",
        "cmdAddPrinter",
        "cmdAddGroup",
        "cmdConfigurePrinter",
        "name",
        "loadPrinterPPD",
        "getServerSettings",
        "saveServerSettings",
        "settings",
        "getRemotePrinters",
        "uri",
        "uriScheme",
        "getRecommendedDrivers",
        "deviceId",
        "makeAndModel",
        "deviceUri",
        "clearRemotePrinters",
        "clearRecommendedDrivers",
        "isIPPCapable",
        "isSCPAvailable",
        "remotePrinters",
        "QVariantList",
        "recommendedDrivers",
        "serverSettings",
        "serverSettingsLoaded",
        "shareConnectedPrinters",
        "allowPrintingFromInternet",
        "allowRemoteAdmin",
        "allowUserCancelAnyJobs"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'requestError'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'serverSettingsChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'serverStopped'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'serverStarted'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'remotePrintersLoaded'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'recommendedDriversLoaded'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ppdLoaded'
        QtMocHelpers::SignalData<void(const QVariantMap &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'cmdAddPrinter'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cmdAddGroup'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cmdConfigurePrinter'
        QtMocHelpers::SignalData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'loadPrinterPPD'
        QtMocHelpers::SlotData<void(const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'getServerSettings'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveServerSettings'
        QtMocHelpers::SlotData<void(const QVariantMap &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 19 },
        }}),
        // Slot 'getRemotePrinters'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 }, { QMetaType::QString, 22 },
        }}),
        // Slot 'getRecommendedDrivers'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 24 }, { QMetaType::QString, 25 }, { QMetaType::QString, 26 },
        }}),
        // Slot 'clearRemotePrinters'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clearRecommendedDrivers'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'isIPPCapable'
        QtMocHelpers::MethodData<bool(const QString &)>(29, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 21 },
        }}),
        // Method 'isSCPAvailable'
        QtMocHelpers::MethodData<bool()>(30, 2, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'remotePrinters'
        QtMocHelpers::PropertyData<QVariantList>(31, 0x80000000 | 32, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 4),
        // property 'recommendedDrivers'
        QtMocHelpers::PropertyData<QVariantList>(33, 0x80000000 | 32, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 5),
        // property 'serverSettings'
        QtMocHelpers::PropertyData<QVariantMap>(34, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
        // property 'serverSettingsLoaded'
        QtMocHelpers::PropertyData<bool>(35, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
        // property 'shareConnectedPrinters'
        QtMocHelpers::PropertyData<bool>(36, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
        // property 'allowPrintingFromInternet'
        QtMocHelpers::PropertyData<bool>(37, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
        // property 'allowRemoteAdmin'
        QtMocHelpers::PropertyData<bool>(38, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
        // property 'allowUserCancelAnyJobs'
        QtMocHelpers::PropertyData<bool>(39, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PrinterManager, qt_meta_tag_ZN14PrinterManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PrinterManager::staticMetaObject = { {
    QMetaObject::SuperData::link<KQuickConfigModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14PrinterManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14PrinterManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14PrinterManagerE_t>.metaTypes,
    nullptr
} };

void PrinterManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PrinterManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->requestError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->serverSettingsChanged(); break;
        case 2: _t->serverStopped(); break;
        case 3: _t->serverStarted(); break;
        case 4: _t->remotePrintersLoaded(); break;
        case 5: _t->recommendedDriversLoaded(); break;
        case 6: _t->ppdLoaded((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 7: _t->cmdAddPrinter(); break;
        case 8: _t->cmdAddGroup(); break;
        case 9: _t->cmdConfigurePrinter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->loadPrinterPPD((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->getServerSettings(); break;
        case 12: _t->saveServerSettings((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 13: _t->getRemotePrinters((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 14: _t->getRecommendedDrivers((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 15: _t->clearRemotePrinters(); break;
        case 16: _t->clearRecommendedDrivers(); break;
        case 17: { bool _r = _t->isIPPCapable((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 18: { bool _r = _t->isSCPAvailable();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PrinterManager::*)(const QString & )>(_a, &PrinterManager::requestError, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PrinterManager::*)()>(_a, &PrinterManager::serverSettingsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PrinterManager::*)()>(_a, &PrinterManager::serverStopped, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PrinterManager::*)()>(_a, &PrinterManager::serverStarted, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PrinterManager::*)()>(_a, &PrinterManager::remotePrintersLoaded, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PrinterManager::*)()>(_a, &PrinterManager::recommendedDriversLoaded, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (PrinterManager::*)(const QVariantMap & )>(_a, &PrinterManager::ppdLoaded, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (PrinterManager::*)()>(_a, &PrinterManager::cmdAddPrinter, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (PrinterManager::*)()>(_a, &PrinterManager::cmdAddGroup, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (PrinterManager::*)(const QString & )>(_a, &PrinterManager::cmdConfigurePrinter, 9))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QVariantList*>(_v) = _t->remotePrinters(); break;
        case 1: *reinterpret_cast<QVariantList*>(_v) = _t->recommendedDrivers(); break;
        case 2: *reinterpret_cast<QVariantMap*>(_v) = _t->serverSettings(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->serverSettingsLoaded(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->shareConnectedPrinters(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->allowPrintingFromInternet(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->allowRemoteAdmin(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->allowUserCancelAnyJobs(); break;
        default: break;
        }
    }
}

const QMetaObject *PrinterManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PrinterManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14PrinterManagerE_t>.strings))
        return static_cast<void*>(this);
    return KQuickConfigModule::qt_metacast(_clname);
}

int PrinterManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KQuickConfigModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 19;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void PrinterManager::requestError(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void PrinterManager::serverSettingsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void PrinterManager::serverStopped()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void PrinterManager::serverStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void PrinterManager::remotePrintersLoaded()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void PrinterManager::recommendedDriversLoaded()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void PrinterManager::ppdLoaded(const QVariantMap & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void PrinterManager::cmdAddPrinter()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void PrinterManager::cmdAddGroup()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void PrinterManager::cmdConfigurePrinter(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}
QT_WARNING_POP
